//+------------------------------------------------------------------+
//|                                        EzSimplGrid.mq4 
//|                      Copyright  2010, HadarFx Software Corp.
//|                      http://hadarfx.blogspot.co.il
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, HadarFx "
#property link      "http://hadarfx.blogspot.co.il"

#include <stdlib.mqh>
//#include <ezdraw.mqh>
#define SIMPLEGRID_VERSION "1.1.0.3"
#define EA_NAME "Simple Grid"


// input parameters for strategy
#property stacksize 64000

// global consts
#define OP_IGNORE	101	
#define DELETE_COLOR    Yellow



extern double strategyDistance = 20;
extern double lotSize = 0.1;
extern double takeProfit = 20;
extern double stopLoss = 0;
extern int	  strategyMagicNumber = 1;
extern int	  SEND_ORDER_PRICE_RETRIES = 10;
extern int	  CLOSE_ORDER_RETRIES = 10;
extern double mySlippage = 3;
extern double	  minimalTakeProfit = 50;
extern double growFactor = 1;
extern int	  numberOfPendingPositions = 5;
extern int    startegyDirection = OP_IGNORE; // no direction set yet
extern bool	  stopOnDuplicates = false;


 int    ATRPeriod = 14;
 int	ATRAVG_PERIOD = 1440;
 double ATRMulti = 2;
 int	  changePricePeriod = 0;

 bool isRunning = true;



// strategy Global variables
int oldstartegyDirection = OP_IGNORE; 


// our local data
int ordersArray[500];     //strartegy orders
int ordersArraySize = 0;  // number of strategy Orders

int pendingOrdersArray[500];     //strartegy orders
int pendingOrdersArraySize = 0;  // number of strategy Orders

int changePricePeriodStart = 0;

bool myPrintMessages = false;

void myPrint(string str)
{
	if(myPrintMessages)
		Print(str);
}

// helper function for filling our array with only relevant orders

void fillOrdersArray()
{
	ordersArraySize = 0;
	pendingOrdersArraySize = 0;

	int posTotal = OrdersTotal();
	for(int i = posTotal-1;i>=0;i--)
	{
		if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
		{
			if(OrderSymbol() == Symbol() && OrderCloseTime() == 0 && OrderMagicNumber() == strategyMagicNumber)
			{
				if(OrderType() == OP_BUY || OrderType() == OP_SELL)
				{
					ordersArray[ordersArraySize] = OrderTicket();
					ordersArraySize++;
				}
				if(OrderType() == OP_BUYLIMIT || OrderType() == OP_SELLLIMIT)
				{
					pendingOrdersArray[pendingOrdersArraySize] = OrderTicket();
					pendingOrdersArraySize++;
				}
			}
		}
	}
}

bool MyRefreshRates()
{
	bool retValue = RefreshRates();
	return (retValue);
}


void DrawHLine(string lineName,double value1,color lineColor= White)
{
//	if(!IsVisible(	)) return;
	if(StringLen(lineName)>=64)
		lineName = StringSubstr(lineName,0,32);
	ObjectDelete(lineName);	
	ObjectCreate(lineName, OBJ_HLINE,0, CurTime(),value1);
	ObjectSet(lineName, OBJPROP_COLOR, lineColor);
	ObjectSet(lineName, OBJPROP_STYLE, STYLE_DASHDOTDOT);
}

void setStrategyDirection()
{
	oldstartegyDirection = startegyDirection ;
	startegyDirection = OP_IGNORE;
	
	
	if(ordersArraySize == 0)
	{
			if(oldstartegyDirection == OP_BUY)
				startegyDirection = OP_SELL;
			if(oldstartegyDirection == OP_SELL)
				startegyDirection = OP_BUY;
	}
	else
		if(OrderSelect(ordersArray[0],SELECT_BY_TICKET,MODE_TRADES)==true)
		{
			if(OrderType() == OP_BUY)
			{
				startegyDirection = OP_BUY;
				oldstartegyDirection = OP_SELL;
			}

			if( OrderType() == OP_SELL)
			{
				startegyDirection = OP_SELL;
				oldstartegyDirection = OP_BUY;
			}
		}
}

int getClosestPosition()
{
	double orderMinDistance = 100000; // max Distance
	int closestPositionTicket = -1;

	if(ordersArraySize == 1)
		if(OrderSelect(ordersArray[0],SELECT_BY_TICKET,MODE_TRADES)==true)
			return (OrderTicket());
		else 
			return (-1);

	double buyOrderPrice =100000 ;
	double sellOrderPrice =0 ;

	for(int i = ordersArraySize-1;i>=0;i--)
	{
		if(OrderSelect(ordersArray[i],SELECT_BY_TICKET,MODE_TRADES)==true)
		{
			if(OrderSymbol() == Symbol() && OrderCloseTime() == 0 && OrderMagicNumber() == strategyMagicNumber)
			{
				double dist = 0;
				if(OrderType() == OP_BUY)
				{
					if(buyOrderPrice > OrderOpenPrice()) 
					{
						buyOrderPrice  = OrderOpenPrice();
						closestPositionTicket = OrderTicket();
					}

				}

				if( OrderType() == OP_SELL)
				{
					if(sellOrderPrice < OrderOpenPrice()) 
					{
						sellOrderPrice  = OrderOpenPrice();
						closestPositionTicket = OrderTicket();
					}
				}
			}
		}
	}

	return( closestPositionTicket);
}


double GetTickValue()
{
	return (MarketInfo(Symbol(), MODE_TICKVALUE));
}


int MyOrderSend(string symbol, int cmd, double volume, double price, int slippage, double stoploss, double tp, string comment, int magic=0, datetime expiration=0, color arrow_color=CLR_NONE)
{
	int numberOfRetries = 0;

	int currentOrderTicket = -1;
	

	while (currentOrderTicket == -1 && numberOfRetries < SEND_ORDER_PRICE_RETRIES )
	{
		if (stoploss < 0)
			stoploss = 0;
		if(cmd==OP_BUY)
		{
			price = Ask;
			if (tp != 0)
				tp =  price + calcPriceDistance(takeProfit);
			if(stoploss != 0)
				stoploss = price - calcPriceDistance(stopLoss);
		}
		if(cmd==OP_SELL)
		{
			price = Bid;
			if (tp != 0)
				tp =  price - calcPriceDistance(takeProfit);
			if(stoploss != 0)
				stoploss = price + calcPriceDistance(stopLoss);
		}
		
		if (tp !=0)
		{
			if(cmd==OP_BUY || cmd == OP_BUYLIMIT)
				if (tp-price < calcPriceDistance(minimalTakeProfit))
					tp = price + calcPriceDistance(minimalTakeProfit);
			if(cmd==OP_SELL || cmd == OP_SELLLIMIT)
				if (price - tp < calcPriceDistance(minimalTakeProfit))
					tp = price - calcPriceDistance(minimalTakeProfit);
		}
		if(arrow_color != CLR_NONE)
		{
			if(cmd == OP_BUY)
				arrow_color = Blue;
			else if (cmd == OP_BUYLIMIT)
				arrow_color = Purple;
			else if (cmd == OP_SELLLIMIT)
				arrow_color = Orange;
			else
				arrow_color = Red;
		}

		//MyRefreshRates();
		if(NormalizeDouble(getMinPosDistFromPrice(price),Digits) < NormalizeDouble(calcPriceDistance(strategyDistance),Digits))
		{
			Print("Price to close to other Position");
			return (-1);
		}


		currentOrderTicket = OrderSend( symbol,  cmd,  volume, NormalizeDouble(price,Digits),  slippage, NormalizeDouble( stoploss,Digits),NormalizeDouble( tp,Digits),  comment,  magic,  expiration,  arrow_color);

		if(currentOrderTicket >= 0)
		{
			MyRefreshRates();
			if(OrderSelect(currentOrderTicket,SELECT_BY_TICKET,MODE_TRADES)) 
			{
				myPrint("Opened A new Order: "+OrderTicket() + " , "+OrderOpenPrice());
			}
		}
		else															    
		{
			numberOfRetries ++;
			int error = GetLastError();
			Print("MyOrderSend() error: "+ error + " (" + cmd + "," + volume + "," + price+ "," + slippage+ "," + stoploss+ "," + tp+ "," +comment
				+") trying again ! retry number: " + numberOfRetries);
			switch(error)
			{
				case 4107:
				case 130:
				default:
					MyRefreshRates();
					switch(cmd)
					{
					case OP_BUYLIMIT:
					case OP_BUY:
						price -= calcPriceDistance(strategyDistance);
						if (tp != 0)
							tp =  price + calcPriceDistance(takeProfit);
						if(stoploss != 0)
							stoploss = price - calcPriceDistance(stopLoss);
						break;
					case OP_SELLLIMIT:
					case OP_SELL:
						price += calcPriceDistance(strategyDistance);
						if (tp != 0)
							tp =  price - calcPriceDistance(takeProfit);
						if(stoploss != 0)
							stoploss = price + calcPriceDistance(stopLoss);
						break;
					}
				}
			}
		}
	return (currentOrderTicket);
}




// initialization function
double AvgAtr = 0;
int	init()
{
	Print("Welcome To " +EA_NAME+" EA. version: " + SIMPLEGRID_VERSION);

	if(startegyDirection == 101)
		startegyDirection = 0;
	if(startegyDirection == 0)
		oldstartegyDirection = 1;
	if(startegyDirection == 1)
		oldstartegyDirection = 0;
	changePricePeriodStart = TimeCurrent();
	AvgAtr = iATR(NULL,0,ATRAVG_PERIOD,0);
	Print ("Starting SimpleGrid, Average ATR: " + AvgAtr);
	return(0);
}



int deinit()
{
	return(0);
}

double calcPriceDistance(double distance)
{
	return (distance/GetTickValue()*Point);
}


double getMinPosDistFromPrice(double price)
{
	double minDistance = 100000;
	for(int i = OrdersTotal()-1;i>=0;i--)
	{
		if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
		{
			if(OrderSymbol() == Symbol() && OrderCloseTime() == 0 && OrderMagicNumber() == strategyMagicNumber)
			{
				if(minDistance > MathAbs(price - OrderOpenPrice()) )
					minDistance = MathAbs(price - OrderOpenPrice());
			}
		}
	}
	return (minDistance);
}


bool MyOrderClose( int ticket, double lots, double price, int slippage, color Color=White)
{
	static int lastClosingTime = 0;
	bool retValue  = false;
	int numberOfTries = CLOSE_ORDER_RETRIES;
	double orderProfit = 0;

	double oldOrderPrice = 0;
	double oldOrderLots = 0;
	double oldOrderOpenTime = 0;
	int oldOrderType = 0;
	if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)== true)
	{
		oldOrderPrice = OrderOpenPrice();
		oldOrderLots = OrderLots();
		orderProfit = OrderProfit();
		oldOrderOpenTime = OrderOpenTime();
		oldOrderType = OrderType();
	}
	else
		return (false);



	while(!retValue && numberOfTries > 0)
	{
		numberOfTries--;
		
		retValue = OrderClose(ticket,NormalizeDouble(lots,Digits),NormalizeDouble(price,Digits) ,slippage,Color);
		if(!retValue)   
		{
			int errorCode = GetLastError();
			myPrint("Error: "+errorCode+" Closing Order: #"+ticket +" lot: " + lots+ " price: " + price);
//ERR_TRADE_TIMEOUT	128	Trade timeout.
//ERR_INVALID_PRICE	129	Invalid price.
//ERR_INVALID_STOPS	130	Invalid stops.
//ERR_INVALID_TRADE_VOLUME	131	Invalid trade volume.
//ERR_MARKET_CLOSED	132	Market is closed.
//ERR_TRADE_DISABLED	133	Trade is disabled.
//ERR_NOT_ENOUGH_MONEY	134	Not enough money.
//ERR_PRICE_CHANGED	135	Price changed.
//ERR_OFF_QUOTES	136	Off quotes.
//ERR_BROKER_BUSY	137	Broker is busy.
//ERR_REQUOTE	138	Requote.

			if(errorCode == 4108)
			{
				//ObSetProperty(ObGetIndexByTicket(ticket),OBI_STATE,OB_STATE_EMPTY);
				//UpdateTicket2(ticket);
			}
			if(errorCode == 138 || errorCode == 129 || errorCode == 130)
			{
				MyRefreshRates();
				switch(oldOrderType)
				{
				case OP_BUY:
					if(price-Bid < 0)
						numberOfTries = 0;
					price = Bid;
					break;
				case OP_SELL:
					if(Ask-price < 0)
						numberOfTries = 0;
					price = Ask;
					break;
				}
			}
			
		}
		else
		{
			lastClosingTime = TimeCurrent();

			double timeOpened = (TimeCurrent() - oldOrderOpenTime )/ 86400;
			
			if(myPrintMessages)
			{

				if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_HISTORY)== true)
				{
					string type = "buy";
					if(OrderType() == OP_SELL)
						type = "sell";
					string tempString = "[Close] "+OrderTicket()+
						" "+TimeToStr(OrderOpenTime(),TIME_DATE|TIME_SECONDS)+
						" "+type+
						" "+OrderLots()+
						" "+Symbol()+
						" "+OrderOpenPrice()+
						" ClosePrice: " + OrderClosePrice()+"(" +price+
						") Profit: "+ OrderProfit()+
						"("+ orderProfit+
						") Spread: "+ MarketInfo(Symbol(),MODE_SPREAD)+
						" balance: " + AccountBalance();
					myPrint(tempString);
				}
			}
		}
	}

	return (retValue);
}
void	checkIfToCloseTakeProfit()
{
	for(int i=0;i< ordersArraySize;i++)
	{
		if(OrderSelect(ordersArray[i],SELECT_BY_TICKET,MODE_TRADES)==true)
		{
			
			if(OrderType() == OP_BUY)
			{
				if(Bid - OrderOpenPrice() > calcPriceDistance(takeProfit) )
				{
					Print("Closing TP: " + OrderTicket()+","+OrderLots()+"," + Bid);
					MyOrderClose(OrderTicket(),OrderLots(),Bid,mySlippage,Blue);
				}
			}
			if(OrderType() == OP_SELL)
			{
				if(OrderOpenPrice() - Ask > calcPriceDistance(takeProfit) )
				{
					Print("Closing TP: " + OrderTicket()+","+OrderLots()+"," + Bid);
					MyOrderClose(OrderTicket(),OrderLots(),Ask,mySlippage,Red);
				}
			}
		}
	}
}

int tempPositionsArray[200];

void checkForDuplicatePosition()
{
	int tempPositionArraySize = 0;

	for(int i = OrdersTotal()-1;i>=0;i--)
	{
		if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
		{
			if(OrderSymbol() == Symbol() && OrderCloseTime() == 0 && OrderMagicNumber() == strategyMagicNumber)
			{
				tempPositionsArray[tempPositionArraySize]  = OrderTicket();
				tempPositionArraySize++;
			}
		}
	}

	double iPrice = 0;
	double jPrice = 0;

	for( i = 0;i<tempPositionArraySize-1;i++)
	{
		for(int j = i+1;j<tempPositionArraySize;j++)
		{
			if(OrderSelect(tempPositionsArray[i],SELECT_BY_TICKET,MODE_TRADES)==true)
			{
				iPrice = OrderOpenPrice();
				if(OrderSelect(tempPositionsArray[j],SELECT_BY_TICKET,MODE_TRADES)==true)
				{
					jPrice = OrderOpenPrice();

					double startegyDist = NormalizeDouble( calcPriceDistance(strategyDistance),Digits);
					double priceDist = NormalizeDouble(MathAbs(iPrice-jPrice),Digits);
					if( startegyDist*95/100 > priceDist )// give or take slippage
					{
						//Print("WE HAVE DUPLICATE POSITIONS: ");
						Print("WE HAVE DUPLICATE POSITIONS: " + (tempPositionsArray[i])+ ","+(tempPositionsArray[j])+" dist= " +DoubleToStrMorePrecision(priceDist,15) + ", startegy Dist = " + DoubleToStrMorePrecision(startegyDist,15));
						if(stopOnDuplicates)
							isRunning = false;
					}
				}
			}
		}
	}


}

int	start()
{
	if(!isRunning)
		return;
	// first we fill our orders array to be able to ignore other positions in the account
	fillOrdersArray();
	checkIfToCloseTakeProfit();
	fillOrdersArray();
	setStrategyDirection();
	myPrint("Startegy Direction: " + startegyDirection + " old Direction: " + oldstartegyDirection);

	double currentATR = iATR(NULL,0,ATRPeriod,0);

	checkForDuplicatePosition();

	if(ordersArraySize == 0) // OPEN FIRST POSITION
	{
		//AvgAtr = iATR(NULL,0,ATRAVG_PERIOD,0);

		//if(currentATR > ATRMulti*AvgAtr)
		//	return;
		double price = 0;
		double takeProfitPrice = 0;
		double stopLossPrice = 0;
		if(startegyDirection == OP_BUY)
		{
			price = Ask;
			takeProfitPrice = price + calcPriceDistance(takeProfit);
			stopLossPrice = price - calcPriceDistance(stopLoss);
		}
		if(startegyDirection == OP_SELL)
		{
			price = Bid;
			takeProfitPrice = price - calcPriceDistance(takeProfit);
			stopLossPrice = price + calcPriceDistance(stopLoss);
		}
		if (stopLoss == 0)
			stopLossPrice = 0;
		myPrint("Trying to open new position: price: "+ price + " tp: " + takeProfitPrice + " sl: " + stopLossPrice); 
		MyOrderSend(Symbol(),startegyDirection,lotSize,price,mySlippage,stopLossPrice,takeProfitPrice,"simple grid startegy",strategyMagicNumber);

		int orderType = startegyDirection;

		if(pendingOrdersArraySize == 0)
		{
		for(int numberOfPos0 = 0;numberOfPos0 < numberOfPendingPositions;numberOfPos0++)
		{
			if(startegyDirection == OP_BUY)
				{
					price -= calcPriceDistance(strategyDistance);
					takeProfitPrice = price + calcPriceDistance(takeProfit);
					stopLossPrice = price - calcPriceDistance(stopLoss);
					orderType = OP_BUYLIMIT;
				}

			if(startegyDirection == OP_SELL)
				{
					price += calcPriceDistance(strategyDistance);
					takeProfitPrice = price - calcPriceDistance(takeProfit);
					stopLossPrice = price + calcPriceDistance(stopLoss);
					orderType = OP_SELLLIMIT;
				}
			
			myPrint("Placing a limit price: " + price + " tp: " + takeProfitPrice + " sl: " + stopLossPrice); 
			if (stopLoss == 0)
				stopLossPrice = 0;
			MyOrderSend(Symbol(),orderType,MathPow(growFactor,1)*lotSize,price,mySlippage,stopLossPrice,takeProfitPrice,"simple grid startegy order limit",strategyMagicNumber,CLR_NONE,1);
		}
		}
		return;
	}
	else if(ordersArraySize != 0)// Verify we have our limits placed
	{
		int limitsPlaced = 0;
		int closestPosition = getClosestPosition();

		if(closestPosition == -1) // 
		{
			Print("Error, Should have been atleast one open position");
			return;
		}
		double closestPrice = 0;
		double minimumGridDist = calcPriceDistance(strategyDistance)* numberOfPendingPositions;

		if(OrderSelect(closestPosition,SELECT_BY_TICKET,MODE_TRADES)==true)
		{
			closestPrice = OrderOpenPrice();
		}
		else
		{
			myPrint("error getting position info: " + closestPosition);
			return;
		}

		int numberOfLimitPositions = 0;


		for(int i = OrdersTotal()-1;i>=0;i--)
		{
			if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
			{
				if(OrderSymbol() == Symbol() && OrderCloseTime() == 0 && OrderMagicNumber() == strategyMagicNumber)
				{
					if(startegyDirection == OP_BUY)
					{
						
						if(OrderType() == OP_SELLLIMIT)
						{
							OrderDelete(OrderTicket(),DELETE_COLOR);
						}
						if(OrderType() == OP_BUYLIMIT)
						{
							if(OrderOpenPrice() > Ask) 
							{
								OrderDelete(OrderTicket(),DELETE_COLOR);
							}
							else if( MathAbs(OrderOpenPrice() -Ask) < minimumGridDist)
								limitsPlaced++;

						}
					}
					else if(startegyDirection == OP_SELL)
					{
						if(OrderType() == OP_BUYLIMIT)
						{
							OrderDelete(OrderTicket(),DELETE_COLOR);
						}
						if(OrderType() == OP_SELLLIMIT)
						{
							if(OrderOpenPrice() < Bid) 
							{
								OrderDelete(OrderTicket(),DELETE_COLOR);
							}
							else if(MathAbs(OrderOpenPrice() -Bid) < minimumGridDist)
								limitsPlaced++;
						}
					}
				}
			}
		}
		


		if(limitsPlaced < numberOfPendingPositions)
		{
			myPrint("Not enough Limits placed"); 
			if(myPrintMessages)
			{
				for( i = OrdersTotal()-1;i>=0;i--)
				{
					if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)
					{
						myPrint("currentOrders " + OrderTicket() + " type: " + OrderType());
					}
				}	
			}
			orderType = -1;

			price = closestPrice;
			for(int numberOfPos = 0;numberOfPos < numberOfPendingPositions;numberOfPos++)
			{
				if(startegyDirection == OP_BUY)
				{
					price -= calcPriceDistance(strategyDistance);
					takeProfitPrice = price + calcPriceDistance(takeProfit);
					stopLossPrice = price - calcPriceDistance(stopLoss);
					orderType = OP_BUYLIMIT;
				}

				if(startegyDirection == OP_SELL)
				{
					price += calcPriceDistance(strategyDistance);
					takeProfitPrice = price - calcPriceDistance(takeProfit);
					stopLossPrice = price + calcPriceDistance(stopLoss);
					orderType = OP_SELLLIMIT;
				}

				if(NormalizeDouble(getMinPosDistFromPrice(price),Digits) < NormalizeDouble(calcPriceDistance(strategyDistance),Digits))
						orderType = -1;

				if(orderType != -1 && orderType !=  OP_IGNORE)
				{
					if (stopLoss == 0)
						stopLossPrice = 0;

					myPrint("Placing a limit price: " + price + " tp: " + takeProfitPrice + " sl: " + stopLossPrice); 

					MyOrderSend(Symbol(),orderType,MathPow(growFactor,ordersArraySize)*lotSize,price,mySlippage,stopLossPrice,takeProfitPrice,"simple grid startegy order limit",strategyMagicNumber,CLR_NONE,1);
				}
			}
		}
	}
}	

